import json


class BaseDomain(object):
    @classmethod
    def send(cls, req):
        cls.ws.send(json.dumps(req))
        return cls.ws.recv()


def get_command(domain_name, command_name):
    def send_command(cls, **kwargs):
        return cls.send({
            'id': 0,
            'method': '{0}.{1}'.format(domain_name, command_name),
            'params': kwargs,
        })

    return send_command

def DomainFactory(domain_name, cmds):
    klass = type(str(domain_name), (BaseDomain,), {})

    for c in cmds:
        command = get_command(domain_name, c['name'])
        setattr(klass, c['name'], classmethod(command))

    return klass



from __future__ import absolute_import

import json
import base64

try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen

from websocket import create_connection

from cproto.domains.factory import DomainFactory


class CProto(object):

    def __init__(self, host='127.0.0.1', port=9222):
        res = urlopen('http://{0}:{1}/json'.format(host, port))

        url = json.loads(res.read())[0]['webSocketDebuggerUrl']
        self.ws = create_connection(url)

        with open('cproto/resources/protocol.json', 'r') as f:
            data = json.loads(f.read())

            for d in data['domains']:
                domain_name = d['domain']

                # Build Domain Class from protocol
                DomainClass = DomainFactory(domain_name, d['commands'])

                # Set WebSocket attribute
                setattr(DomainClass, 'ws', self.ws)

                # Set Domain's Class as a property
                setattr(self, domain_name, DomainClass)

    def close(self):
        self.ws.close()



from setuptools import setup, find_packages
from codecs import open
from os import path


ROOT_DIR = path.abspath(path.dirname(__file__))
EXCLUDE_FROM_PACKAGES = ['examples', 'tests', 'tools']

with open(path.join(ROOT_DIR, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='cproto',
    version='0.1.2',
    description='Chrome Debugging Protocol client',
    long_description=long_description,
    url='https://github.com/asyne/cproto',
    author='Evan K.',
    author_email='cproto+asyne.inout@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: Browsers',
        'Topic :: Software Development :: Debuggers',
        'Topic :: Software Development :: Testing',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='chrome debug client',
    packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    install_requires=['websocket-client'],
)
